subroutine CalcEnergy(Pos, M, L, rc, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000., r0 = 1.
    real(8), dimension(Dim) :: rij
    real(8) :: Lj, Spring
    real(8) :: d, Shift, d1
    integer :: i, j
    LJ = 0.
    Spring = 0.
    Shift = -4. * (rc**(-12) - rc**(-6)) 
    do i = 0, NAtom - 1
	do j = i + 1, NAtom - 1
	    rij = Pos(j,:) - Pos(i,:)
	    rij = rij - L * dnint(rij / L)
	    if (j == i + 1 .and. mod(j, M) > 0) then
		d1 = sqrt(sum(rij * rij))
		Spring = Spring + 0.5 * k * ((d1 - r0) * (d1 - r0))
	    else
		d = sqrt(sum(rij * rij))
		if (d > rc) then
		    cycle
		endif
		LJ = LJ + 4. * (d**(-12) - d**(-6)) + Shift
	    endif
	PEnergy = LJ + Spring
	enddo
    enddo
end subroutine

subroutine CalcEnergyForces(Pos, M, L, rc, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8) :: LJEnergy, SprEnergy
    real(8), intent(out), dimension(0:NAtom-1, 0: Dim-1) :: Forces
!f2py intent(in, out, inplace) :: Forces
    real(8), parameter :: k = 3000., r0 = 1.
    real(8), dimension(Dim) :: rij, Fij
    real(8) :: d, d1, Shift
    integer :: i, j
    SprEnergy = 0.
    LJEnergy = 0.
    Forces = 0.
    Shift = -4. * (rc**(-12) - rc**(-6))
    do i = 0, NAtom - 1
	do j = i + 1, NAtom - 1
   	    rij = Pos(j,:) - Pos(i,:)
	    rij = rij - L * dnint(rij / L)
	    if (j == i + 1 .and. mod(j, M) > 0) then
		d1 = sqrt(sum(rij * rij))
		SprEnergy = SprEnergy + 0.5 * k * ((d1 - r0) * (d1 - r0))
		Fij = rij * k * (d1 - r0) / d1
		Forces(i,:) = Forces(i,:) + Fij
		Forces(j,:) = Forces(j,:) - Fij
	    else
		d = sqrt(sum(rij * rij))
		if (d > rc) then
		    cycle
		endif
		LJEnergy = LJEnergy + 4. * (d**(-12) - d**(-6)) + Shift
		Fij = rij * (-48. * d**(-14) + 24. * d**(-8))
		Forces(i,:) = Forces(i,:) + Fij
		Forces(j,:) = Forces(j,:) - Fij
	    endif
	PEnergy = LJEnergy + SprEnergy
	enddo
    enddo
end subroutine

subroutine VVIntegrate(Pos, Vel, Accel, M, L, rc, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in) :: L, rc, dt
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out,inplace) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, M, L, rc, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine